﻿#include "Obelisk.h"

#include "CMainController.h"
#include "CScriptEngine.h"
#include "CGameController.h"
#include "CGameState.h"
#include "COverworld.h"
#include "CCombat.h"

using namespace std;
using namespace Obelisk;



CGameController::CGameController(IObelisk* aObelisk, CMainController* aOwner)
{
	mObelisk = aObelisk;
	mOwner = aOwner;

	glFrontFace(GL_CW);
	glDisable(GL_DEPTH_TEST);

	mCursor = false;

	mScriptEngine = new CScriptEngine(mObelisk, this);
	mScriptEngine->GameStart();

	mGame = new CGameState(aObelisk, this);

	mState = gsOverworld;
	mOverworld = new COverworld(aObelisk, this);
	mCombat = NULL;

	mGameOver = false;
	mWin = false;

	mFirstCombat = true;
}



CGameController::~CGameController()
{
	delete mOverworld;
	if (mCombat != NULL)
		delete mCombat;
	delete mGame;
	delete mScriptEngine;
}



bool CGameController::Update()
{
	mCursor = false;
	if (mOwner->GetPaused())
		return false;
	if (mScriptEngine->Update())
	{
		mCursor = true;
		return false;
	}
	if (mGameOver)
	{
		if (!mWin && (mObelisk->MouseButtonClicked(false) || mObelisk->IsKeyDown(kEnter)))
			return true;
	}
	else
		mGame->Update();

	switch (mState)
	{
		case gsOverworld:
			if (!mGameOver)
				mOverworld->Update();
			break;
		case gsCombat:
			if (mCombat->Update())
			{
				delete mCombat;
				mCombat = NULL;
			}
			break;
	}
	return false;
}



void CGameController::Render()
{
	switch (mState)
	{
		case gsOverworld:
			mOverworld->Render();
			break;
		case gsCombat:
			mCombat->Render();
			break;
	}

	if (mGameOver)
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glScalef(1, -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		CVector BackV0(-1, -1, 0);
		CVector BackV1(1, -1, 0);
		CVector BackV2(1, 1, 0);
		CVector BackV3(-1, 1, 0);
		mObelisk->SetAlpha(0.7f);
		mObelisk->TriColour(BackV0, BackV1, BackV2, CColour(0.3f, 0.3f, 0.3f));
		mObelisk->TriColour(BackV0, BackV2, BackV3, CColour(0.3f, 0.3f, 0.3f));
		mObelisk->SetAlpha(1.0f);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		mObelisk->SetSpriteViewport(1024, 768);
		mCursor = true;
		if (!mWin)
		{
			mObelisk->DrawText(SL("Game Over"), 512 - 77, 200);
			mObelisk->DrawText(mGameOverMessage, 512 - mGameOverSize / 2, 400);
			mObelisk->DrawText(SL("Click Or Press Enter To Restart"), 512 - 481 / 2, 550);
		}
		else
		{
			DrawRect(0, 0, 1024, 768, CColour(0, 0, 0));
			mObelisk->DrawText(SL("The End"), 512 - 50, 300);
			mObelisk->DrawText(SL("(Close The Window To Quit)"), 512 - 415 / 2, 550);
		}
	}
	else
	{
		mScriptEngine->Render();
	}

	if (mCursor)
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		mObelisk->SetSpriteViewport(1024, 768);
		int CursorX;
		int CursorY;
		mObelisk->GetCursorPos(CursorX, CursorY);
		if (CursorX >= 0 && CursorX < 1024 && CursorY >= 0 && CursorY < 768)
			mObelisk->DrawSprite(mObelisk->GetTexture(SL("Crosshair.bmp")), CursorX,
					CursorY, -1, -1, 0.5, 0.5);
	}
}



void CGameController::DrawRect(float aX0, float aY0, float aX1, float aY1, const CColour& aColour)
{
	CVector V0(aX0, aY0, 0);
	CVector V1(aX1, aY0, 0);
	CVector V2(aX1, aY1, 0);
	CVector V3(aX0, aY1, 0);
	mObelisk->TriColour(V0, V1, V2, aColour);
	mObelisk->TriColour(V0, V2, V3, aColour);
}



void CGameController::Combat(CIsland* aIsland)
{
	mState = gsCombat;
	mCombat = new CCombat(mObelisk, this, aIsland);

	if (mFirstCombat)
	{
		mFirstCombat = false;
		mScriptEngine->FirstCombat();
	}
}



void CGameController::FinishCombat()
{
	mState = gsOverworld;
}



void CGameController::GameOver(CString aMessage, int aSize)
{
	mState = gsOverworld;
	mGameOver = true;
	mGameOverMessage = aMessage;
	mGameOverSize = aSize;
}



void CGameController::WIN()
{
	mState = gsOverworld;
	mGameOver = true;
	mWin = true;
}



bool CGameController::IsEnterPressed()
{
	return mOwner->IsEnterPressed();
}



bool CGameController::IsEscapePressed()
{
	return mOwner->IsEscapePressed();
}



void CGameController::FirstInRange()
{
	mScriptEngine->FirstInRange();
}



void CGameController::FirstRank()
{
	mScriptEngine->FirstRank();
}



void CGameController::FirstIslandWon()
{
	mScriptEngine->FirstIslandWon();
}



void CGameController::LevelWon(int aLevel)
{
	mScriptEngine->LevelWon(aLevel);
}