﻿#include <vector>
#include "Obelisk.h"

#include "CMainController.h"
#include "CGameController.h"

using namespace std;
using namespace Obelisk;



IMainController* CreateMainController(IObelisk* aObelisk)
{
	return new CMainController(aObelisk);
}



CMainController::CMainController(IObelisk* aObelisk)
{
	mObelisk = aObelisk;
	mObelisk->SetCaption(SL("Island Commander"));
	mObelisk->SetSize(1024, 768);
	mObelisk->SetDirectory(SL("Island Commander LD19"));
	mState = false;
	mPaused = false;
	mGame = NULL;

	mEnterDownLast = false;
	mEscapeDownLast = false;
}



CMainController::~CMainController()
{
	if (mGame != NULL)
		delete mGame;
}



void CMainController::Initialise()
{
	mObelisk->LoadTexture(SL("Title.bmp"), false);
	vector<CString> Textures;
	Textures.push_back(SL("BoatLeft.bmp"));
	Textures.push_back(SL("BoatRight.bmp"));
	Textures.push_back(SL("Crosshair.bmp"));
	Textures.push_back(SL("Select.bmp"));
	Textures.push_back(SL("HeliLeft0.bmp"));
	Textures.push_back(SL("HeliLeft1.bmp"));
	Textures.push_back(SL("HeliRight0.bmp"));
	Textures.push_back(SL("HeliRight1.bmp"));
	Textures.push_back(SL("Explosion.bmp"));
	Textures.push_back(SL("TurretLeft.bmp"));
	Textures.push_back(SL("TurretRight.bmp"));
	Textures.push_back(SL("Cmdr.bmp"));
	Textures.push_back(SL("You.bmp"));
	Textures.push_back(SL("BadGuy.bmp"));
	for (vector<CTexture>::size_type i = 0; i < Textures.size(); i++)
		mObelisk->LoadTexture(Textures[i], true);
	mFont0 = mObelisk->LoadFontImage(SL("Comic Sans MS_16BI.bmp"));
	mObelisk->StartTimer();
}



void CMainController::OnNotify(int aMessage, int aData)
{
}



void CMainController::OnTimer()
{
	if (!mState)
	{
		mObelisk->BeginFrame();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		mObelisk->SetSpriteViewport(1024, 768);
		mObelisk->DrawSprite(mObelisk->GetTexture(SL("Title.bmp")), 512, 384, -1, -1, 0.5, 0.5);
		mObelisk->SetFont(mFont0);
		mObelisk->DrawText(SL("Click Or Press Enter To Start"), 512 - 451 / 2, 650);
		mObelisk->EndFrame();

		if (mObelisk->MouseButtonClicked(false) || mObelisk->IsKeyDown(kEnter))
		{
			mState = true;
			mGame = new CGameController(mObelisk, this);
		}
	}
	else
	{
		if (mGame->Update())
		{
			delete mGame;
			mGame = new CGameController(mObelisk, this);
		}
		else
		{
			mObelisk->BeginFrame();
			mGame->Render();
			mObelisk->EndFrame();
		}
	}

	mEnterDownLast = mObelisk->IsKeyDown(kEnter);
	mEscapeDownLast = mObelisk->IsKeyDown(kEscape);
}



void CMainController::OnDeactivate()
{
	mPaused = true;
}



void CMainController::OnActivate()
{
	mPaused = false;
}



void CMainController::OnKeyDown(EKey aKey)
{
}



void CMainController::OnKeyUp(EKey aKey)
{
}



void CMainController::ThreadFinished(IThread* aThread)
{
}



bool CMainController::IsEnterPressed()
{
	return mObelisk->IsKeyDown(kEnter) && !mEnterDownLast;
}



bool CMainController::IsEscapePressed()
{
	return mObelisk->IsKeyDown(kEscape) && !mEscapeDownLast;
}